/*
  aiplayer.h - computer KBalm player

  Copyright 2004 by Richard Smith <kde@metafoo.co.uk>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
*/

#ifndef AIPLAYER_H
#define AIPLAYER_H

#include "kbalm.h"
#include "player.h"
#include "board.h"
#include "infect.h"

#include <list>

namespace KBalmAI
{

class Player;

class Thought
{
public:
	Thought( int level, Balm::Board &board, Balm::Game &game, int turn, int self, int x, int y );
	~Thought();
	
	bool think();
	void bored();
	double value();
	
private:
	void calculateValue();
	
private:
	int _level;
	double _value;
	Balm::Board _board;
	Balm::Game &_game;
	int _turn, _self;
	bool _done;
	std::auto_ptr<Balm::InfectBase> _infect;
	typedef std::list<Thought*> _thoughts_t;
	_thoughts_t _thoughts;
};

struct PossibleMove
{
	PossibleMove( int x, int y, Thought *thought ) : move( x, y ), thought( thought ) {}
	std::pair<int,int> move;
	Thought *thought;
};
	
class Turn : public Balm::Turn
{
	Q_OBJECT
public:
	Turn( Player &player, Balm::Game &game );
	override ~Turn();
	
private slots:
	void think();
	void bored();
	
private:
	typedef std::list<PossibleMove> _moves_t;
	_moves_t _moves;
};

class Player : public Balm::Player
{
public:
	Player( int skill, int number ) : Balm::Player(number), _skill(skill) {}
	override Balm::Turn *createTurn( Balm::Game &game ) { return new Turn( *this, game ); }
	
	int skill() { return _skill; }
private:
	int _skill;
};

} // namespace KBalmAIPlayer

#endif

//kate: indent-mode: csands;
