/*
  board.h - KBalm game board
  
  Copyright 2004 by Richard Smith <kde@metafoo.co.uk>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
*/

#ifndef BOARD_H
#define BOARD_H

#include "common.h"

#include <qobject.h>

#include <vector>
#include <memory>

namespace Balm
{

class Player;
class BoardSignals;

class Direction
{
public:
	enum RandomTag { Random };
	Direction( RandomTag );
		
	enum Name { Up, Right, Down, Left };
	Direction( Name name ) : _name(name) {}
	
	Name name() const { return _name; }
	
	int x() const;
	int y() const;
	
	Direction rotate() const;
	
	bool operator==(const Direction &dir) const { return dir._name == _name; }
	bool operator!=(const Direction &dir) const { return dir._name != _name; }
	
private:
	Name _name;
};

class Node
{
	Direction _dir;
	const Player *_owner;
public:
	Node( Direction dir, const Player *owner = 0 ) : _dir(dir), _owner(owner) {}
	
	const Player *owner() const { return _owner; }
	void setOwner( const Player *owner ) { _owner = owner; }
	
	Direction direction() const { return _dir; }
	void setDirection( Direction dir ) { _dir = dir; }
};

class Board
{
public:
	Board( const Board & );
	Board( int width, int height, std::auto_ptr<BoardSignals> signalz = std::auto_ptr<BoardSignals>( 0 ) );
	~Board();
	
	int width() const;
	int height() const;
	
	BoardSignals *signalz() const;
	
	int score( int player ) const;
	int totalScore() const;
	
	const Node *at( int x, int y ) const;
	bool connected( int x, int y, Direction dir ) const;
	QString playerNumber( int x, int y ) const;
	
	void setOwner( int x, int y, const Player *owner );
	void setDirection( int x, int y, Direction dir );
	void rotate( int x, int y );
	
private:
	std::vector< std::vector< Node > > _nodes;
	std::vector< int > _score;
	std::auto_ptr<BoardSignals> _signals;
};

class BoardSignals : public QObject
{
	Q_OBJECT
	
public:
	BoardSignals();
	
signals:
	void scoreChanged();
	void nodeChanged( int x, int y );

private:	
	friend class Board;
};

}

#endif
