/*
  boardwidget.h - widget displaying a KBalm Board
  
  Copyright 2004 by Richard Smith <kde@metafoo.co.uk>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
*/

#ifndef BOARDWIDGET_H
#define BOARDWIDGET_H

#include "common.h"

#include <qscrollview.h>

#include <memory>

class KBalmPixmapCache;

namespace Balm
{

class Board;

class BoardWidget : public QScrollView
{
	Q_OBJECT
	
public:
	BoardWidget( QWidget *parent, Board *board, const char *name = 0 );
	override ~BoardWidget();
	
public slots:
	void moveLeft();
	void moveRight();
	void moveUp();
	void moveDown();
	void selectCurrent();
	
signals:
	void nodeSelected( int x, int y );
	
protected:
	override void drawContents( QPainter *p, int x, int y, int w, int h );
	override void contentsMouseReleaseEvent( QMouseEvent *e );

private:
	void showSelection();
	void hideSelection();
	void moveSelection( int xOff, int yOff );
	void ensureNodeVisible( int x, int y );
	
	void drawHConnect( QPainter *p, int x, int y, const QString &player );
	void drawVConnect( QPainter *p, int x, int y, const QString &player );
	
private slots:
	void boardChanged( int x, int y );
	
private:
	Board *_board;
	bool _showSel;
	int _selX, _selY;
	std::auto_ptr<KBalmPixmapCache> _cache;
};

}

#endif
