/*
  game.cpp - a balm game
  
  Copyright 2004 by Richard Smith <kde@metafoo.co.uk>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
*/

#include "game.h"

#include "board.h"
#include "player.h"

#include <kdebug.h>
#include <klocale.h>

namespace Balm
{

Game::Params::Params( QObject *parent, const char *name )
	: _parent(parent), _name(name), _width(20), _height(20)
{
}

void Game::Params::addPlayer( Balm::Player *player )
{
	_players.push_back( player );
}


static void setupPlayerStart( Balm::Game *game, int playerNumber, int x, int y, Direction direction )
{
	if ( Balm::Player *player = game->player( playerNumber ) )
	{
		game->board().setOwner( x, y, player );
		game->board().setDirection( x, y, direction );
	}
}
	
Game::Game( const Params &params )
	: QObject(params._parent, params._name), _players(params._players), _board( new Board( params._width, params._height, std::auto_ptr<BoardSignals>( new BoardSignals ) ) )
{
	// make sure that players start facing one turn off the board (fairness)
	const int w = params._width - 1, h = params._height - 1;
	setupPlayerStart( this, 0, 0, 0, Balm::Direction::Up );
	setupPlayerStart( this, 1, w, h, Balm::Direction::Down );
	setupPlayerStart( this, 2, w, 0, Balm::Direction::Right );
	setupPlayerStart( this, 3, 0, h, Balm::Direction::Left );
	
	_player = _players.size();
	nextPlayer();
}

Game::~Game()
{
	_turn.reset();
	for ( uint n = 0; n < _players.size(); ++n )
		delete _players[n];
}
	
Player *Game::player(int n)
{
	if ( n < 0 || n >= _players.size() )
		return 0;
	return _players[n];
}

void Game::nextPlayer()
{
	int oldPlayer = _player;
	
	for ( uint n = 0; n < players(); ++n )
		if ( board().score( n ) == 0 )
			_players[n]->setStatus( i18n( "Eliminated" ) );
	
	do
	{
		if ( ++_player >= _players.size() )
			_player = 0;
	} while ( board().score( _players[_player]->number() ) == 0 );
	
	if ( oldPlayer == _player )
	{
		_turn.reset();
		_players[_player]->setStatus( i18n( "Winner" ) );
		emit gameOver( _players[_player] );
	}
	else
	{	
		_turn.reset( _players[_player]->createTurn( *this ) );
		connect( _turn.get(), SIGNAL( finished() ), SLOT( nextPlayer() ) );
	}
}

}

#include "game.moc"
//kate: indent-mode: csands;
