/*
  game.h - a balm game
  
  Copyright 2004 by Richard Smith <kde@metafoo.co.uk>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
*/

#ifndef GAME_H
#define GAME_H

#include "common.h"

#include <qobject.h>

#include <vector>
#include <memory>

namespace Balm
{

class Player;
class Board;
class Turn;

class Game : public QObject
{
	Q_OBJECT
	
public:
	class Params;
	Game( const Params &params );
	override ~Game();
	
	Board &board() { return *_board; }
	
	int players() { return _players.size(); }
	Player *player(int n);
	
signals:
	void gameOver( Player *winner );
	
private slots:
	void nextPlayer();
	
private:
	std::vector<Player*> _players;
	Board *_board;
	uint _player;
	std::auto_ptr<Turn> _turn;
};

class Game::Params
{
public:
	Params( QObject *parent, const char *name = 0 );
	void setSize( int width, int height ) { _width = width; _height = height; }
	void addPlayer( Player *player );
	
private:
	friend class Game;
	
	QObject *_parent;
	const char *_name;
	int _width, _height;
	std::vector<Player*> _players;
};
	
}

#endif
