/*
  infect.cpp - infection propogator
  
  Copyright 2004 by Richard Smith <kde@metafoo.co.uk>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
*/

#include "infect.h"

#include "board.h"

namespace Balm
{

//BEGIN class Infect

InfectBase::InfectBase( Board &board, int x, int y )
	: _board( board )
{
	_player = board.at( x, y )->owner();
	_targets.push_back( std::make_pair( x, y ) );
}

bool InfectBase::process()
{
	if ( _targets.empty() )
		return true;
	
	int x = _targets.front().first, y = _targets.front().second;
	_targets.pop_front();
	
	const Node *source = _board.at( x, y );
	Direction lookIn = Direction::Up;
	for ( uint n = 0; n < 4; ++n )
	{
		int nx = x + lookIn.x(), ny = y + lookIn.y();
		if ( const Node *curr = _board.at( nx, ny ) )
		{
			if ( curr->owner() != _player && _board.connected( x, y, lookIn ) )
			{
				_board.setOwner( nx, ny, _player );
				_targets.push_back( std::make_pair( nx, ny ) );
			}
		}
		lookIn = lookIn.rotate();
	}
	return false;
}


Infect::Infect( Board &board, int x, int y )
	: QObject( board.signalz() ), InfectBase( board, x, y )
{
}

Infect::~Infect()
{
}

void Infect::slotProcess()
{
	if ( process() )
	{
		emit finished();
		deleteLater();
	}
}
	
}

#include "infect.moc"
//kate: indent-mode: csands;
