/*
  player.cpp - KBalm player
  
  Copyright 2004 by Richard Smith <kde@metafoo.co.uk>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
*/

#include "player.h"

#include "game.h"
#include "board.h"
#include "infect.h"

#include <klocale.h>

#include <qtimer.h>

namespace Balm
{

//BEGIN class Player

Player::Player( int number )
	: QObject( 0 ), _num( number ), _status( i18n("Waiting") )
{
}

Player::~Player()
{
}

//END class Player


//BEGIN class Turn

Turn::~Turn()
{
	setStatus( i18n("Waiting") );
}

void Turn::move( int x, int y )
{
	if ( _moved ) return;
	if ( board().at( x, y )->owner() != &player() ) return;
	
	setStatus( i18n("Moving...") );
	
	board().rotate( x, y );
	_moved = true;
	
	Infect *infect = createInfect( x, y );
	connect( infect, SIGNAL( finished() ), SIGNAL( finished() ) );
	
	QTimer *timer = new QTimer( this );
	connect( timer, SIGNAL( timeout() ), infect, SLOT( slotProcess() ) );
	timer->start( 50 );
}

Infect *Turn::createInfect( int x, int y )
{
	return new Infect( board(), x, y );
}

Board &Turn::board()
{
	return game().board();
}

//END class Turn

} // namespace Balm

#include "player.moc"
//kate: indent-mode: csands;
