/*
  player.h - KBalm player
  
  Copyright 2004 by Richard Smith <kde@metafoo.co.uk>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
*/

#ifndef PLAYER_H
#define PLAYER_H

#include "common.h"

#include <qobject.h>
#include <qcolor.h>

namespace Balm
{

class Game;
class Turn;
class Board;
class Infect;

class Player : public QObject
{
	Q_OBJECT
	
public:
	Player( int number );
	override ~Player();
	
	QColor color() const;
	int number() const { return _num; }
	
	virtual Turn *createTurn( Game &game ) = 0;
	
	QString status() { return _status; }
	void setStatus( const QString &str )
	{
		_status = str;
		emit status( str );
	}
	
signals:
	void status( const QString &str );
	
private:
	QString _status;
	int _num;
};

class Turn : public QObject
{
	Q_OBJECT
	
public:
	Turn( Player &player, Game &game ) : QObject(&player), _player(player), _game(game), _moved(false) {}
	override ~Turn();
	
	Player &player() { return _player; }
	Board &board();
	Game &game() { return _game; }
	
	void setStatus( const QString &str ) { player().setStatus( str ); }
	
protected:
	virtual Infect *createInfect( int x, int y );
	
protected slots:
	void move( int x, int y );
	
signals:
	void finished();
	
private:
	Player &_player;
	Game &_game;
	bool _moved;
};

}

#endif
