/*
  scorepanel.cpp - widget displaying balm scores
  
  Copyright 2004 by Richard Smith <kde@metafoo.co.uk>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
*/

#include "scorepanel.h"

#include "board.h"
#include "game.h"
#include "player.h"
#include "pixmapcache.h"

#include <klocale.h>

#include <qvbox.h>
#include <qhbox.h>
#include <qlabel.h>
#include <qpixmap.h>
#include <qstring.h>


KBalmStatusLabel::KBalmStatusLabel( QWidget *parent, Balm::Player *player, const char *name )
	: QLabel( player->status(), parent, name )
{
	connect( player, SIGNAL( status(const QString&) ), SLOT( setText(const QString&) ) );
}


KBalmScorePanel::KBalmScorePanel( QWidget *parent, Balm::Game *game, const char *name )
	: QVBox( parent, name ), _game( game )
{
	KBalmPixmapCache cache;
	
	setFrameStyle( StyledPanel | Sunken );
	setMargin( 2 );
	setSpacing( 3 );
	setSizePolicy( QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed ) );
	
	for ( int n = 0; n < game->players(); ++n )
	{
		Balm::Player *player = game->player( n );
		QHBox *box = new QHBox( this );
		QLabel *label = new QLabel( box );
		label->setPixmap( cache.pixmap( "pics/node_" + QString::number( player->number() + 1 ) + ".png" ) );
		
		QVBox *vbox = new QVBox( box );
		QLabel *score = new QLabel( vbox );
		_labels.push_back( score );
		
		new KBalmStatusLabel( vbox, player );
		
		box->setStretchFactor( new QLabel( box ), 1 );
		
		(new QFrame( this ))->setFrameStyle( HLine | Sunken );
	}
	
	QHBox *box = new QHBox( this );
	QLabel *label = new QLabel( box );
	label->setPixmap( cache.pixmap( "pics/node_0.png" ) );
	
	QLabel *score = new QLabel( box );
	_labels.push_back( score );
	
	box->setStretchFactor( new QLabel( box ), 1 );
	
	scoreChanged();
	
	connect( _game->board().signalz(), SIGNAL( scoreChanged() ), SLOT( scoreChanged() ) );
}

KBalmScorePanel::~KBalmScorePanel()
{
}
	
void KBalmScorePanel::scoreChanged()
{
	for ( int n = 0; n < _game->players(); ++n )
	{
		Balm::Player *player = _game->player( n );
		int score = _game->board().score( player->number() );
		_labels[n]->setText( i18n("One node controlled", "%n nodes controlled", score) );
	}
	int cells = _game->board().width() * _game->board().height();
	_labels[_game->players()]->setText( i18n("One node unoccupied", "%n nodes unoccupied", cells - _game->board().totalScore()) );
}
	
#include "scorepanel.moc"
//kate: indent-mode: csands;
